#!/usr/local/bin/wish -f

source server.tcl

global par
global log
global ind;  set ind  0
global procnr; set procnr 512 
global slack; set slack 8
global proto; set proto log
global recs; set recs 4
global mods; set mods 1
global s; set s 1; global si; set si 1
global x0; set x0 50
global y0; set y0 50
global ideal; set ideal 37

proc init {} {
  global ind; global ideal; global proto; global recs
  global s; global i; global x0; global y0
  set font1 -Adobe-Helvetica-Medium-R-Normal--*-120-*

  wm title . "PRAM simulator" 

  frame .ctrl

  frame .ctrl.procnr
  label .ctrl.procnr.label -font $font1 -text "Processors? "
  radiobutton .ctrl.procnr.4 -text 4 -variable procnr -value 4   
  radiobutton .ctrl.procnr.64 -text 64 -variable procnr -value 64  
  radiobutton .ctrl.procnr.128 -text 128 -variable procnr -value 128  
  radiobutton .ctrl.procnr.256 -text 256 -variable procnr -value 256
  radiobutton .ctrl.procnr.512 -text 512 -variable procnr -value 512 
  radiobutton .ctrl.procnr.1024 -text 1024 -variable procnr -value 1024 
  pack .ctrl.procnr.label -side left -expand 1 -anchor w 
  pack .ctrl.procnr.4 .ctrl.procnr.64 .ctrl.procnr.128\
       .ctrl.procnr.256 .ctrl.procnr.512 .ctrl.procnr.1024 \
       -side left -fill x -anchor e

  frame .ctrl.slack
  label .ctrl.slack.label -font $font1 -text "Slackness? "
  radiobutton .ctrl.slack.1  -text  1 -variable slack -value  1
  radiobutton .ctrl.slack.2  -text  2 -variable slack -value  2
  radiobutton .ctrl.slack.4  -text  4 -variable slack -value  4
  radiobutton .ctrl.slack.8  -text  8 -variable slack -value  8
  radiobutton .ctrl.slack.16 -text 16 -variable slack -value 16 
  radiobutton .ctrl.slack.32 -text 32 -variable slack -value 32 
  pack .ctrl.slack.label -side left -expand 1 -anchor w
  pack .ctrl.slack.1 .ctrl.slack.2 .ctrl.slack.4 .ctrl.slack.8 \
         .ctrl.slack.16 .ctrl.slack.32 -side left -fill x -anchor e

  frame .ctrl.proto
  label .ctrl.proto.label -font $font1 -text "Protocol? "
  radiobutton .ctrl.proto.greedy -text greedy -variable proto -value greedy
  radiobutton .ctrl.proto.log -text "log" -variable proto -value log
  radiobutton .ctrl.proto.sqrt -text "sqrt" -variable proto -value sqrt
  radiobutton .ctrl.proto.lin -text "lin" -variable proto -value lin 
  radiobutton .ctrl.proto.sqr -text "sqr" -variable proto -value square
  menubutton .ctrl.proto.menu -text Menu -menu .ctrl.proto.menu.menu  
  menu .ctrl.proto.menu.menu
    .ctrl.proto.menu.menu add radiobutton \
       -label "log f" -variable proto -value log
    .ctrl.proto.menu.menu add radiobutton \
       -label "sqrt f" -variable proto -value sqrt
    .ctrl.proto.menu.menu add radiobutton \
       -label "f,logp" -variable proto -value flogp
    .ctrl.proto.menu.menu add radiobutton \
       -label "lin" -variable proto -value lin
    .ctrl.proto.menu.menu add radiobutton \
       -label flogf -variable proto -value flogf
    .ctrl.proto.menu.menu add radiobutton \
       -label "f*f" -variable proto -value square
    .ctrl.proto.menu.menu add radiobutton \
       -label "2**f" -variable proto -value exp
  pack .ctrl.proto.label -side left -expand 1 -anchor w
  pack .ctrl.proto.greedy .ctrl.proto.log .ctrl.proto.sqrt .ctrl.proto.lin \
       .ctrl.proto.sqr .ctrl.proto.menu -side left -fill x -anchor e

  frame .ctrl.recs
  label .ctrl.recs.label -font $font1 -text "Receivers? "
  radiobutton .ctrl.recs.1  -text  1 -variable recs -value  1
  radiobutton .ctrl.recs.2  -text  2 -variable recs -value  2
  radiobutton .ctrl.recs.4  -text  4 -variable recs -value  4
  radiobutton .ctrl.recs.8  -text  8 -variable recs -value  8
  radiobutton .ctrl.recs.16 -text 16 -variable recs -value 16 
  pack .ctrl.recs.label -side left -expand 1 -anchor w
    pack .ctrl.recs.1 .ctrl.recs.2 .ctrl.recs.4 .ctrl.recs.8 .ctrl.recs.16 \
         -side left -fill x -anchor e

  frame .ctrl.mods
  label .ctrl.mods.label -font $font1 -text "Module size? "
  radiobutton .ctrl.mods.1  -text  1 -variable mods -value  1
  radiobutton .ctrl.mods.2  -text  2 -variable mods -value  2
  radiobutton .ctrl.mods.4  -text  4 -variable mods -value  4
  radiobutton .ctrl.mods.8  -text  8 -variable mods -value  8
  radiobutton .ctrl.mods.16 -text 16 -variable mods -value 16 
  pack .ctrl.mods.label -side left -expand 1 -anchor w
    pack .ctrl.mods.1 .ctrl.mods.2 .ctrl.mods.4 .ctrl.mods.8 .ctrl.mods.16 \
         -side left -fill x -anchor e

  frame .ctrl.go
  label .ctrl.go.label -font $font1 -text "Run now! "
  button .ctrl.go.go -text "simulate" -command {
    set high $s 
    for {set i $high} {$i > 0} {set i [expr $i-1]} {
      p {restart}; p {compile(pram)}; p {debugoff}
      p {resetplot}
      p "simulate($procnr,$slack,$proto,$recs,$mods)"
    }
  }
  label .ctrl.go.times -text "*"
  entry .ctrl.go.sims -width 2 -relief sunken -textvariable s
  bind  .ctrl.go.sims <Return> \
        {set sims $s; update_log "Repeat\ $s\ times\n"}
  button .ctrl.go.debug -text "dump" -command {
    reset_plot; set ideal [expr exp(-1*$recs)]; p {compile(pram)}; 
    p {debugon}; p "simulate($procnr,$slack,$proto,$recs,$mods)"
  }
  button .ctrl.go.stop -text "stop" -command "destroy ."
  pack .ctrl.go.label -side left -expand 1 -anchor w
  pack .ctrl.go.go .ctrl.go.times .ctrl.go.sims .ctrl.go.debug .ctrl.go.stop \
       -side left

  pack .ctrl.procnr .ctrl.slack .ctrl.proto .ctrl.recs .ctrl.mods .ctrl.go \
       -side top -fill x

  frame .log
  text .log.txt -width 60 -height 6 -bd 2 -bg white \
	  -yscrollcommand ".log.scroll set" -setgrid true  
  scrollbar .log.scroll -command ".log.txt yview"
  pack .log.scroll -side right -fill y
  pack .log.txt -side left  -fill both

  canvas .plot -relief raised -width 520 -height 200
# (x0,y0) is at point (x=0,y=100)
  .plot create line $x0 [expr $y0+100] [expr $x0+(3*128)] [expr $y0+100] \
               -width 2
  .plot create line $x0 [expr $y0+100] $x0 $y0 -width 2
  .plot create text [expr $x0+30] [expr $y0-20] \
       -text "Percentage of sendable/tried/received packets at each round" \
       -anchor w 
  for {set i 0} {$i <= 12} {incr i} {
      set x [expr {$x0 + ($i*30)}]
      .plot create line $x [expr $y0+103] $x [expr $y0+99] -width 1
      .plot create text $x [expr $y0+104] -text [expr 10*$i] -anchor n 
  }
  for {set i 0} {$i <= 10} {incr i 1} {
      set y [expr {$y0+100 - ($i*10)}]
      .plot create line [expr $x0-3] $y [expr $x0+1] $y -width 1
  }
  .plot create line [expr $x0-4] [expr $y0+100] [expr $x0+1] [expr $y0+100] \
               -width 3
  .plot create text [expr $x0-4] [expr $y0+100] -text "0" -anchor e
  .plot create line [expr $x0-4] [expr $y0+50] [expr $x0+1] [expr $y0+50] \
               -width 3
  .plot create text [expr $x0-4] [expr $y0+50] -text " 50" -anchor e
  .plot create line [expr $x0-4]  $y0 [expr $x0+1] $y0 -width 3
  .plot create text [expr $x0-4]  $y0 -text "100" -anchor e

  pack .ctrl .log .plot
}

proc update_log {pm} {
#  .log.txt insert end " "
  .log.txt insert end $pm
  .log.txt see end
}

proc update_plot3 {y1 y2 y3} {
  global ind; global ideal; global x0; global y0
  set ind [expr 1+($ind%128)]
  set x [expr {$x0 + (3*$ind)}]
  set y00 [expr $y0+100]
  set y01 [expr {[expr $y0+100] - $y1}]
  set y02 [expr {[expr $y0+100] - $y2}]
  set y03 [expr {[expr $y0+100] - $y3}]
  .plot create line $x $y00 $x $y01 -width 3 -fill black
  .plot create line $x $y01 $x $y02 -width 3 -fill red
  .plot create line $x $y02 $x $y03 -width 3 -fill green
  .plot create line $x $y03 $x $y0  -width 3 -fill white
  .plot create line [expr $x-1] [expr $y0+100-$ideal] \
                    [expr $x+2] [expr $y0+100-$ideal] -width 1 -fill blue
}

proc update_sims {sims} {
  global s
  set s $sims
}

proc reset_plot {} {
  global ind; global ideal; global recs; global x0; global y0
  set ind 0
  .plot create line [expr $x0+192] [expr $y0+100] [expr $x0+192] $y0 \
                    -width 384 -fill white
  set ideal [expr round(100.0*pow(0.3678795,(1.0/$recs)))]
  .plot create line $x0            [expr $y0+100-$ideal] \
                    [expr $x0+384] [expr $y0+100-$ideal] \
                    -width 1 -fill blue
  .plot create text [expr $x0+388] [expr $y0+100-$ideal] \
                    -text "1/e^1/$recs" -fill blue -anchor w
}

init
start_prolog
