% BinProlog server for tcl/tk

:-module(prolog).
:-['../src/wam'].
:-end_module.

:-op(200,fx,('$')).

main:-prolog_server.

prolog_server:-
	init_server,
	repeat,
	  tcl_in(X,Vs),
        react(X,Vs),
	!.
		
init_server:-
	interactive(no),
	cwrite('# Prolog process ready'),nl,tcl_prompt.

qtest:-
  repeat,
    cwrite('% '),flush,
    read_term(T,Vs),
    react(T,Vs),
  !.

react(end_of_file,_):-!.
react(call_prolog(X),_):-!,X,!,fail.
react(query_prolog(X),Vs):-query_prolog(X,Vs),fail.

query_prolog(X,Vs):-X,nl,member(A,Vs),write(A),nl,fail.
query_prolog(_,_):-tcl_prompt.

tcl_prompt :- call_tcl(puts('-nonewline "% "')).

call_tcl(Line):-
	cwrite('call_tcl '),tcl_out(Line).
	
tcl_out([X|Xs]):-!,out_list([X|Xs]).
tcl_out(T):-out_term(T).

out_list(L):-member(T,L),out_term(T),fail.
out_list(_).

out_term(T):-pl2tcl(T),!.
out_term(T):-errmes(unable_to_write,T).

tcl_in(T):-tcl_in(T,_).

tcl_in(T,Vs):-read_term(L,Vs),!,in1(L,T).

in1([X|Xs],T):-!,member(T,[X|Xs]).
in1(T,T).

pl2tcl(T):-pl2tcl(T,L),writelist(L),nl.

pl2tcl(X,L):-pl2tcl(X,L,[]).

pl2tcl(X)-->{var(X)},!,{errmes(unexpected_in(tcl),var(X))}.
pl2tcl(X)-->{atomic(X)},!,[X].
pl2tcl('$'(X))-->!,{ val(X,X,V);V='$'(X)},!,[V].
pl2tcl([X|Xs])-->!,['{'],pl2tcl(X),tcl_cmd(Xs),['}'].
pl2tcl({Xs})-->!,['{'],pl2tcl(Xs),['}'].
pl2tcl((X,Xs))-->!,pl2tcl(X),pl2tcl(Xs).
pl2tcl(T)-->{T=..[F|Xs]},begin_script(F,Xs,Ys),tcl_cmd(Ys),end_script(F).

begin_script(script,Xs,Xs)-->!,['['].
begin_script(F,Xs,[F|Xs])-->['{'].

end_script(script)-->!,[']'].
end_script(_)-->['}'].

tcl_cmd([X|Xs])-->!,pl2tcl(X),tcl_cmd(Xs).
tcl_cmd([])-->[].

writelist(L):-
  member(X,L),
  cwrite1(X),cwrite(' '),
  fail.
writelist(_).

cwrite1('$'(X)):-!,cwrite('$'),cwrite(X).
cwrite1(X):-float(X),!,write(X).
cwrite1(X):-cwrite(X).

